@@ -14,5 +14,15 @@ logging.config.dictConfig(config)
 
 
 def get_logger(name: str) -> Logger:
+    """
+    Creates a logger object for a given name.
+
+    Args:
+        name (str): Name of the logger.
+
+    Returns:
+        Logger: Logger object with given name.
+    """
+
     logger = getLogger(name)
     return logger
\ No newline at end of file
